@@ -1,10 +1,10 @@
-## ---- include = FALSE----------------------------------------------------
+## ---- include = FALSE---------------------------------------------------------
 knitr::opts_chunk$set(collapse = T, comment = "#>")
 options(tibble.print_min = 4L, tibble.print_max = 4L)
 library(dplyr)
 set.seed(1014)
 
-## ------------------------------------------------------------------------
+## -----------------------------------------------------------------------------
 library(Lahman)
 
 batting <- Lahman::Batting %>%
@@ -15,7 +15,7 @@ batting <- Lahman::Batting %>%
 
 players <- batting %>% group_by(playerID)
 
-## ---- eval = FALSE-------------------------------------------------------
+## ---- eval = FALSE------------------------------------------------------------
 #  # For each player, find the two years with most hits
 #  filter(players, min_rank(desc(H)) <= 2 & H > 0)
 #  # Within each player, rank each year by the number of games played
@@ -31,67 +31,67 @@ players <- batting %>% group_by(playerID)
 #  # For each, player compute a z score based on number of games played
 #  mutate(players, G_z = (G - mean(G)) / sd(G))
 
-## ------------------------------------------------------------------------
+## -----------------------------------------------------------------------------
 x <- c(1, 1, 2, 2, 2)
 
 row_number(x)
 min_rank(x)
 dense_rank(x)
 
-## ------------------------------------------------------------------------
+## -----------------------------------------------------------------------------
 cume_dist(x)
 percent_rank(x)
 
-## ------------------------------------------------------------------------
+## -----------------------------------------------------------------------------
 filter(players, cume_dist(desc(G)) < 0.1)
 
-## ------------------------------------------------------------------------
+## -----------------------------------------------------------------------------
 by_team_player <- group_by(batting, teamID, playerID)
 by_team <- summarise(by_team_player, G = sum(G))
 by_team_quartile <- group_by(by_team, quartile = ntile(G, 4))
 summarise(by_team_quartile, mean(G))
 
-## ------------------------------------------------------------------------
+## -----------------------------------------------------------------------------
 x <- 1:5
 lead(x)
 lag(x)
 
-## ---- results = "hide"---------------------------------------------------
+## ---- results = "hide"--------------------------------------------------------
 # Compute the relative change in games played
 mutate(players, G_delta = G - lag(G))
 
-## ---- results = "hide"---------------------------------------------------
+## ---- results = "hide"--------------------------------------------------------
 # Find when a player changed teams
 filter(players, teamID != lag(teamID))
 
-## ------------------------------------------------------------------------
+## -----------------------------------------------------------------------------
 df <- data.frame(year = 2000:2005, value = (0:5) ^ 2)
 scrambled <- df[sample(nrow(df)), ]
 
-wrong <- mutate(scrambled, running = cumsum(value))
+wrong <- mutate(scrambled, prev_value = lag(value))
 arrange(wrong, year)
 
-right <- mutate(scrambled, running = order_by(year, cumsum(value)))
+right <- mutate(scrambled, prev_value = lag(value, order_by = year))
 arrange(right, year)
 
-## ---- eval = FALSE-------------------------------------------------------
+## ---- eval = FALSE------------------------------------------------------------
 #  filter(players, cumany(G > 150))
 
-## ------------------------------------------------------------------------
+## -----------------------------------------------------------------------------
 x <- 1:10
 y <- 10:1
 order_by(y, cumsum(x))
 
-## ---- eval = FALSE-------------------------------------------------------
+## ---- eval = FALSE------------------------------------------------------------
 #  filter(players, G > mean(G))
 #  filter(players, G < median(G))
 
-## ---- eval = FALSE-------------------------------------------------------
+## ---- eval = FALSE------------------------------------------------------------
 #  filter(players, ntile(G, 2) == 2)
 
-## ------------------------------------------------------------------------
+## -----------------------------------------------------------------------------
 mutate(players, career_year = yearID - min(yearID) + 1)
 
-## ------------------------------------------------------------------------
+## -----------------------------------------------------------------------------
 mutate(players, G_z = (G - mean(G)) / sd(G))
 
