@@ -57,7 +57,11 @@ public:
   }
 
   inline void clear(SEXP mask_resolved) {
-    Rf_defineVar(symbol, R_UnboundValue, mask_resolved);
+    try {
+      Rcpp::Environment env(mask_resolved);
+      const char* nm = CHAR(PRINTNAME(symbol));
+      env.remove(nm);
+    } catch (...) { }
   }
 
   // summary accessor
@@ -152,6 +156,9 @@ private:
                             );
     MARK_NOT_MUTABLE(value);
 
+    // store it in the mask_resolved environment
+    Rf_defineVar(symbol, value, mask_resolved);
+
     return value;
   }
 
