@@ -2,12 +2,12 @@
 #define dplyr_tools_all_na_H
 
 template <int RTYPE>
-inline bool all_na_impl(const Vector<RTYPE>& x) {
+inline bool all_na_impl(const Rcpp::Vector<RTYPE>& x) {
   return all(is_na(x)).is_true();
 }
 
 template <>
-inline bool all_na_impl<REALSXP>(const NumericVector& x) {
+inline bool all_na_impl<REALSXP>(const Rcpp::NumericVector& x) {
   return all(is_na(x) & !is_nan(x)).is_true();
 }
 
